@@ -2,6 +2,8 @@ import numpy as np
 from ruleskit import Activation
 import pytest
 
+Activation.FORCE_STAT = True
+
 
 @pytest.mark.parametrize(
     "vector, cs, ca, i, n",
@@ -106,3 +108,23 @@ def test_add(vector1, vector2, add_vector):
     act1 = Activation(vector1)
     act2 = Activation(vector2)
     np.testing.assert_equal((act1 + act2).raw, add_vector)
+
+
+def test_sizes():
+    act = Activation(np.array([1, 0, 1]))
+    assert act.sizeof_raw > 0
+    assert act.sizeof_integer > 0
+    assert act.sizeof_compressed_array > 0
+    assert act.sizeof_compressed_str > 0
+
+
+def test_times():
+    act = Activation(np.array([1, 0, 1]))
+    assert act.time_compress > -1
+    assert act.time_decompress > -1
+    assert act.time_conversions_to_int > -1
+    assert act.time_conversions_from_int > -1
+    assert act.decompressions > 0
+    assert act.compressions > 0
+    assert act.conversions_to_int > 0
+    assert act.conversions_from_int > 0
